#include "stdafx.h"
#include "VCAddinUtils.h"

#ifdef _PSEUDO_DEBUG
#include <stdarg.h>
#endif // _PSEUDO_DEBUG

CMDIFrameWnd* GetMainFrameWnd()
{
	CWinApp* pApp = AfxGetApp();
	return pApp ? (CMDIFrameWnd*)pApp->GetMainWnd() : NULL;
}

CWnd* FindMDIClientWnd()
{
	CMDIFrameWnd* pMainFrameWnd = GetMainFrameWnd();
	if ( !pMainFrameWnd || !IsWindow(*pMainFrameWnd) )
		return NULL;
	HWND hWndMDIClient = ::FindWindowEx(*pMainFrameWnd, NULL, WC_VC_MIDICLIENT, NULL);
	return CWnd::FromHandle(hWndMDIClient);
}

HWND FindScrollBarCtrl( HWND hAfxMDIFrameWnd, int nBar )
{
	if ( hAfxMDIFrameWnd )
	{
		HWND hScroll	= NULL;
		while ( hScroll = FindWindowEx(hAfxMDIFrameWnd, hScroll, WC_SCROLLBAR, NULL) )
		{
			DWORD dwStyle = GetWindowLong(hScroll, GWL_STYLE);
			if ( SB_VERT == nBar )
			{
				if ( (dwStyle & SBS_VERT) )
				{
					return hScroll;
				}
			}
			else if ( SB_HORZ == nBar && (dwStyle & SBS_HORZ) )
			{
				return hScroll;
			}
		}
	}
	return NULL;
}

BOOL IsMDIChildWnd( HWND hWnd )
{
	DWORD dwExtendedStyle = (DWORD)GetWindowLong(hWnd, GWL_EXSTYLE);
	return (dwExtendedStyle & WS_EX_MDICHILD);
}

BOOL IsWindowMatchClassName( HWND hWnd, LPCTSTR lpcszClassName )
{
	TCHAR szClassName[WNDCLASSNAME_SIZE] = {0};
	GetClassName(hWnd, szClassName, WNDCLASSNAME_SIZE);
	return _tcsicmp(szClassName, lpcszClassName) == 0;
}

BOOL IsAfxMDIFrameWnd(HWND hWnd)
{
	return IsWindowMatchClassName(hWnd, WC_AFX_MDI_FRAMEWND);
}

BOOL IsScrollBarCtrl(HWND hWnd)
{
	return IsWindowMatchClassName(hWnd, WC_SCROLLBAR);
}

void VCTrace( LPCTSTR lpszFormat, ... )
{
#ifdef _PSEUDO_DEBUG
	va_list args;
	va_start(args, lpszFormat);

	CString strDbgStr;
	strDbgStr.FormatV(lpszFormat, args);

	OutputDebugString(strDbgStr);
#endif // _PSEUDO_DEBUG
}

CString GetWndTraceInfo( HWND hWnd )
{
#ifdef _PSEUDO_DEBUG
	TCHAR szTitle[MAX_PATH];
	GetWindowText(hWnd, szTitle, MAX_PATH);

	TCHAR szClass[WNDCLASSNAME_SIZE] = {0};
	GetClassName(hWnd, szClass, WNDCLASSNAME_SIZE);

	CString strInfo;
	strInfo.Format(_T("[%p] \"%s\" %s"), hWnd, szTitle, szClass);
	return strInfo;
#else
	return CString();
#endif // _PSEUDO_DEBUG
}
