// VCLineNumberModule.cpp: implementation of the CVCLineNumberModule class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VCLineNumberModule.h"
#include "VCLineNumberWnd.h"
#include "VCAddinManager.h"


//////////////////////////////////////////////////////////////////////
// CVCLineNumberModule
//////////////////////////////////////////////////////////////////////

CVCLineNumberModule::CVCLineNumberModule()
{
	m_stSettings.m_clrTextColor		= RGB(0,0,0);
	m_stSettings.m_clrBKColor		= RGB(255,255,255);
	m_hWndProcHook					= NULL;
}

CVCLineNumberModule::~CVCLineNumberModule()
{
	
}

void CVCLineNumberModule::Enable( BOOL bEnable /*= TRUE*/ )
{
	IVCExtendModule::Enable(bEnable);
	if ( bEnable )
	{
		UpdateFont();
		CreateLineNumberWndForExistingMDIChildWnds();
		Hook();
	}
	else
	{
		UnHook();
		DestroyAllLineNumberWnds();
	}
}

BOOL CVCLineNumberModule::LoadSettings()
{
	return TRUE;
}

BOOL CVCLineNumberModule::SaveSettings()
{
	return TRUE;
}

#define HK_VC_SOURCE_WINDOW_FORMAT		_T("Software\\Microsoft\\DevStudio\\6.0\\Format\\Source Window")
#define HK_VC_FONT_FACE_SUBKEY			_T("FontFace")
#define HK_VC_FONT_SIZE_SUBKEY			_T("FontSize")
#define HK_VC_SOURCE_TEXT_SUBKEY		_T("Text")

#define LINE_NUMBER_DEFAULT_FONT_FACE	_T("Courier New")
#define LINE_NUMBER_DEFAULT_FONT_SIZE	11

BOOL CVCLineNumberModule::UpdateFont()
{
	HKEY hKey	= NULL;
	LOGFONT lf	= {0};
	BOOL bRet	= FALSE;
	BOOL bReadFontFace = FALSE;
	BOOL bReadFontSize = FALSE;
	if ( RegOpenKeyEx(HKEY_CURRENT_USER, HK_VC_SOURCE_WINDOW_FORMAT, 0, KEY_READ, &hKey) == ERROR_SUCCESS )
	{
		DWORD nValSize = LF_FACESIZE;
		bReadFontFace = RegQueryValueEx(hKey, HK_VC_FONT_FACE_SUBKEY, NULL, NULL, (LPBYTE)lf.lfFaceName, &nValSize) == ERROR_SUCCESS;
		
		DWORD dwSize = sizeof(DWORD);
		DWORD dwFontSize = LINE_NUMBER_DEFAULT_FONT_SIZE;
		bReadFontSize = RegQueryValueEx(hKey, HK_VC_FONT_SIZE_SUBKEY, NULL, NULL, (LPBYTE)&dwFontSize, &dwSize) == ERROR_SUCCESS;
		if ( bReadFontSize )
		{
			lf.lfHeight = -MulDiv(dwFontSize, GetDeviceCaps(::GetDC(NULL), LOGPIXELSY), 72);
		}

		dwSize = 12;
		BYTE byColors[12];
		if ( RegQueryValueEx(hKey, HK_VC_SOURCE_TEXT_SUBKEY, NULL, NULL, byColors, &dwSize) == ERROR_SUCCESS )
		{
			m_stSettings.m_clrTextColor	= *(COLORREF*)(BYTE*)byColors;
			m_stSettings.m_clrBKColor	= *(((COLORREF*)(BYTE*)byColors) +1);
		}
		
		RegCloseKey(hKey);
		bRet = TRUE;
	}
	
	if ( !bReadFontFace )
	{
		_tcsncpy(lf.lfFaceName, LINE_NUMBER_DEFAULT_FONT_FACE, LF_FACESIZE);
	}
	if ( !bReadFontSize )
	{
		lf.lfHeight = LINE_NUMBER_DEFAULT_FONT_SIZE;
	}
	
	lf.lfQuality = ANTIALIASED_QUALITY;
	
	m_font.DeleteObject();
	m_font.CreateFontIndirect(&lf);
	
	return bRet;
}

BOOL CVCLineNumberModule::Hook()
{
	if ( !m_hWndProcHook )
	{
		m_hWndProcHook = ::SetWindowsHookEx(WH_CALLWNDPROC, CallWndProcHook, NULL, ::GetCurrentThreadId());
		return m_hWndProcHook != NULL;
	}
	return TRUE;
}

BOOL CVCLineNumberModule::UnHook()
{
	BOOL bRet = TRUE;
	if ( m_hWndProcHook )
	{
		bRet = ::UnhookWindowsHookEx(m_hWndProcHook);
		m_hWndProcHook = NULL;
	}
	return bRet;
}

LRESULT CALLBACK CVCLineNumberModule::CallWndProcHook(int code, WPARAM wParam, LPARAM lParam)
{
	CVCLineNumberModule* pModule = (CVCLineNumberModule*)g_pAddinManager->GetModule(CVCAddinManager::AMT_LINENUMBER);

	LRESULT lResult = CallNextHookEx(pModule->m_hWndProcHook, code, wParam, lParam);

	if ( HC_ACTION == code )
	{
		LPCWPSTRUCT lpcWPST = reinterpret_cast<LPCWPSTRUCT>(lParam);
		if (lpcWPST)
		{
			if ( SBM_SETSCROLLINFO == lpcWPST->message )
			{
				BOOL bRedraw				= lpcWPST->wParam;
				LPSCROLLINFO pScrollInfo	= reinterpret_cast<LPSCROLLINFO>(lpcWPST->lParam);
				if ( pScrollInfo )
				{
					HWND hParentWnd = GetParent(lpcWPST->hwnd);	// AfxMDIFrameWnd42
					if ( hParentWnd && (hParentWnd = GetParent(hParentWnd)) && IsMDIChildWnd(hParentWnd) )
					{
// 						if (pScrollInfo)
// 						{
// 							if ( pScrollInfo->fMask & SIF_RANGE )
// 							{
// 								VCTRACE(_T("Update--\tMin: %d, Max: %d\n"), pScrollInfo->nMin, pScrollInfo->nMax);
// 							}
// 							if ( pScrollInfo->fMask & SIF_POS )
// 							{
// 								VCTRACE(_T("Update--\tPos: %d\n"), pScrollInfo->nPos);
// 							}
// 						}
						pModule->CheckUpdateLineNumberWndForMDIChildWnd(hParentWnd);
					}
				}
			}
		}
	}
	
	return lResult;
}

void CVCLineNumberModule::CreateLineNumberWndForExistingMDIChildWnds()
{
	VCTRACE(_T("Trying to create line number windows for all the existing child frame windows!\n"));
	int nCount = 0;
	CWnd* pMDIClientWnd = FindMDIClientWnd();
	if ( pMDIClientWnd )
	{
		HWND hMDIClientWnd = pMDIClientWnd->GetSafeHwnd();
		HWND hWnd = NULL;
		while ( hWnd = ::FindWindowEx(hMDIClientWnd, hWnd, NULL, NULL) )
		{
			if ( IsMDIChildWnd(hWnd) && !GetLineNumberWndForMDIChildWnd(hWnd) )
			{
				if ( CreateLineNumberWndForMDIChildWnd(hWnd) )
				{
					++nCount;
				}
			}
		}
	}

	VCTRACE(_T("%d line number windows have been created.\n"), nCount);
}

void CVCLineNumberModule::DestroyAllLineNumberWnds()
{
	VCTRACE(_T("Trying to destroy all the line number windows!\n"));
	int nCount = 0;
	CWnd* pMDIClientWnd = FindMDIClientWnd();
	if ( pMDIClientWnd )
	{
		HWND hMDIClientWnd = pMDIClientWnd->GetSafeHwnd();
		HWND hWnd = NULL;
		while ( hWnd = ::FindWindowEx(hMDIClientWnd, hWnd, NULL, NULL) )
		{
			if ( IsMDIChildWnd(hWnd) )
			{
				CVCLineNumberWnd* pLineNumberWnd = GetLineNumberWndForMDIChildWnd(hWnd);
				if (pLineNumberWnd)
				{
					pLineNumberWnd->DestroyWindow();
					++nCount;
				}
			}
		}
	}
	VCTRACE(_T("%d line number windows have been destroyed.\n"), nCount);
}

CVCLineNumberWnd* CVCLineNumberModule::GetLineNumberWndForMDIChildWnd( HWND hMDIChildWnd ) const
{
	CVCLineNumberWnd* pLineNumberWnd = (CVCLineNumberWnd*)CWnd::FromHandle( ::FindWindowEx(hMDIChildWnd, NULL, WC_LINENUMBER, NULL) );
	return pLineNumberWnd;
}

CVCLineNumberWnd* CVCLineNumberModule::CreateLineNumberWndForMDIChildWnd( HWND hMDIChildWnd )
{
	ASSERT( !GetLineNumberWndForMDIChildWnd(hMDIChildWnd) );

	CVCLineNumberWnd* pLineNumberWnd = new CVCLineNumberWnd;

	VERIFY( pLineNumberWnd->Create(hMDIChildWnd) );

	VCTRACE(_T("====== A new line number window was [%p] created for %s ======\n"), pLineNumberWnd->GetSafeHwnd(), GetWndTraceInfo(hMDIChildWnd));

	return pLineNumberWnd;
}

void CVCLineNumberModule::CheckUpdateLineNumberWndForMDIChildWnd( HWND hMDIChildWnd )
{
	CVCLineNumberWnd* pLineNumberWnd = GetLineNumberWndForMDIChildWnd(hMDIChildWnd);
	if ( !pLineNumberWnd )
	{
		pLineNumberWnd = CreateLineNumberWndForMDIChildWnd(hMDIChildWnd);
	}
}


